package com.emmacanhelp.phone_caller

import android.Manifest.permission.CALL_PHONE
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.net.Uri
import android.telephony.TelephonyManager
import android.util.Log
import androidx.core.app.ActivityCompat
import androidx.core.content.ContextCompat
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.PluginRegistry

/** PhoneCallerPlugin */
class PhoneCallerPlugin : FlutterPlugin, MethodCallHandler, ActivityAware,
    PluginRegistry.RequestPermissionsResultListener {
    private val callPhoneRequestCode = 7801
    private lateinit var channel: MethodChannel
    private lateinit var flutterBinding: FlutterPlugin.FlutterPluginBinding
    private lateinit var activityBinding: ActivityPluginBinding
    private lateinit var flutterResult: Result
    private lateinit var phoneNumber: String

    override fun onAttachedToEngine(flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
        channel = MethodChannel(flutterPluginBinding.binaryMessenger, "phone_caller")
        channel.setMethodCallHandler(this)
        flutterBinding = flutterPluginBinding
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        flutterResult = result
        when (call.method) {
            "callNumber" -> {
                var number = call.argument<String>("number")
                number = number?.replace("#", "%23")
                if (number != null && !number.startsWith("tel:")) {
                    number = "tel://$number"
                }
                if (number != null) {
                    phoneNumber = number
                    if (isTelephonyEnabled()) {
                        if (isCallPhonePermissionGranted()) {
                            result.success(callNumber(number))
                        } else {
                            requestsPermission()
                        }
                    } else {
                        phoneFeatureError()
                    }
                } else {
                    numberError()
                }
            }
            else -> {
                result.notImplemented()
            }
        }
    }

    override fun onDetachedFromEngine(binding: FlutterPlugin.FlutterPluginBinding) {
        channel.setMethodCallHandler(null)
    }

    override fun onAttachedToActivity(binding: ActivityPluginBinding) {
        activityBinding = binding
        activityBinding.addRequestPermissionsResultListener(this)
    }

    override fun onDetachedFromActivityForConfigChanges() {
    }

    override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    }

    override fun onDetachedFromActivity() {
        activityBinding.removeRequestPermissionsResultListener(this)
    }

    private fun isTelephonyEnabled(): Boolean =
        ((flutterBinding.applicationContext.getSystemService(Context.TELEPHONY_SERVICE) as TelephonyManager?)?.phoneType
            ?: TelephonyManager.PHONE_TYPE_NONE) != TelephonyManager.PHONE_TYPE_NONE

    private fun isCallPhonePermissionGranted(): Boolean =
        ContextCompat.checkSelfPermission(
            flutterBinding.applicationContext,
            CALL_PHONE
        ) == PackageManager.PERMISSION_GRANTED

    private fun callNumber(number: String): Boolean {
        return try {
            Intent(Intent.ACTION_CALL, Uri.parse(number)).let {
                it.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                flutterBinding.applicationContext.startActivity(it)
            }
            true
        } catch (e: Exception) {
            Log.d("PhoneCaller", "error: " + e.message)
            false
        }
    }

    private fun requestsPermission() {
        ActivityCompat.requestPermissions(
            activityBinding.activity,
            arrayOf(CALL_PHONE),
            callPhoneRequestCode
        )
    }

    override fun onRequestPermissionsResult(
        requestCode: Int,
        permissions: Array<out String>?,
        grantResults: IntArray?
    ): Boolean {
        when (requestCode) {
            callPhoneRequestCode -> {
                val r = grantResults?.firstOrNull()
                if (r != null) {
                    if (r == PackageManager.PERMISSION_GRANTED) {
                        flutterResult.success(callNumber(phoneNumber))
                    } else {
                        permissionError()
                    }
                } else {
                    permissionError()
                }
                return true
            }
        }
        return false
    }

    private fun permissionError() {
        flutterResult.error(
            "1",
            "CALL_PHONE permission not granted",
            "The app needs CALL_PHONE permission to make phone call."
        )
    }

    private fun numberError() {
        flutterResult.error(
            "2",
            "Phone number is missing",
            "A phone number must be provided to make phone call."
        )
    }

    private fun phoneFeatureError() {
        flutterResult.error(
            "0",
            "Phone feature not available",
            "It looks like the device can not make phone call."
        )
    }
}
